/*****************************************************************************
 * Copyright (c) 2013, 2017, 2019 CEA LIST, Christian W. Damus, and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   CEA LIST - Initial API and implementation
 *   Christian W. Damus (CEA) - bug 429242
 *   Christian W. Damus (CEA) - bug 422257
 *   Christian W. Damus (CEA) - bug 437052
 *   Christian W. Damus - bug 436998
 *   Eike Stepper (CEA) - bug 466520
 *   Nicolas FAUVERGUE (ALL4TEC) nicolas.fauvergue@all4tec.net - Bug 496905
 *   Vincent LORENZO (CEA) - bug 507763
 *****************************************************************************/
package org.eclipse.papyrus.cdo.core.resource;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.emf.cdo.CDOObject;
import org.eclipse.emf.cdo.CDOState;
import org.eclipse.emf.cdo.common.protocol.CDOProtocolConstants;
import org.eclipse.emf.cdo.dawn.gmf.util.DawnDiagramUpdater;
import org.eclipse.emf.cdo.eresource.CDOResource;
import org.eclipse.emf.cdo.eresource.CDOResourceNode;
import org.eclipse.emf.cdo.explorer.CDOExplorerUtil;
import org.eclipse.emf.cdo.explorer.checkouts.CDOCheckout;
import org.eclipse.emf.cdo.transaction.CDOTransaction;
import org.eclipse.emf.cdo.util.CDOUtil;
import org.eclipse.emf.cdo.util.CommitException;
import org.eclipse.emf.cdo.view.CDOView;
import org.eclipse.emf.cdo.view.CDOViewInvalidationEvent;
import org.eclipse.emf.cdo.view.CDOViewSet;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.spi.cdo.FSMUtil;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.net4j.util.collection.AbstractFilteredIterator;
import org.eclipse.net4j.util.event.EventUtil;
import org.eclipse.net4j.util.event.IEvent;
import org.eclipse.net4j.util.event.IListener;
import org.eclipse.papyrus.cdo.internal.core.CDOUtils;
import org.eclipse.papyrus.cdo.internal.core.controlmode.CDOControlModeParticipant;
import org.eclipse.papyrus.cdo.internal.core.controlmode.CDOProxyManager;
import org.eclipse.papyrus.infra.core.Activator;
import org.eclipse.papyrus.infra.core.resource.ModelMultiException;
import org.eclipse.papyrus.infra.services.resourceloading.OnDemandLoadingModelSet;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;

/**
 * This is the CDOAwareModelSet type. Enjoy.
 */
public class CDOAwareModelSet extends OnDemandLoadingModelSet {

	@SuppressWarnings("restriction")
	private static final String SCHEME = org.eclipse.emf.cdo.internal.explorer.checkouts.CDOCheckoutViewProvider.SCHEME;

	private static final Set<CDOState> DIRTY_STATES = EnumSet.of(CDOState.NEW, CDOState.DIRTY, CDOState.CONFLICT, CDOState.INVALID_CONFLICT);

	private final ThreadLocal<Boolean> inGetResource = new ThreadLocal<Boolean>();

	private final CDOProxyManager proxyManager = new CDOProxyManager(this);

	private final PapyrusCDOResourceFactory resourceFactory = new PapyrusCDOResourceFactory(this);

	private final Adapter viewSetAdapter = new ViewSetAdapter();

	private final IListener invalidationListener = new InvalidationListener();

	private CDOView view;

	private boolean unloading;


	public CDOAwareModelSet() {
		this.resources = new SafeResourceList();

		Map<String, Object> map = getResourceFactoryRegistry().getProtocolToFactoryMap();
		map.put(CDOProtocolConstants.PROTOCOL_NAME, resourceFactory);
		map.put(SCHEME, resourceFactory);
	}

	@Override
	public final boolean isShardingSupported() {
		return false;
	}

	public final IListener getInvalidationListener() {
		return invalidationListener;
	}

	@Override
	protected Adapter createModificationTrackingAdapter() {
		return new CDOAwareProxyModificationTrackingAdapter();
	}

	@Override
	public EObject getEObject(URI uri, boolean loadOnDemand) {
		if (CDOUtils.isCDOURI(uri)) {
			return getCDOObject(uri, loadOnDemand);
		}

		return super.getEObject(uri, loadOnDemand);
	}

	protected EObject getCDOObject(URI uri, boolean loadOnDemand) {
		if (CDOProxyManager.isCDOProxyURI(uri)) {
			// get a real proxy object, out of thin air, to give CDO the non-null
			// instance that it needs (otherwise the 'non-null constraint' of
			// all kinds of reference lists will be violated)
			return proxyManager.getProxy(uri);
		}

		return super.getEObject(uri, loadOnDemand);
	}

	@Override
	public Resource getResource(URI uri, boolean loadOnDemand) {
		Boolean oldValue = inGetResource.get();
		inGetResource.set(Boolean.TRUE);

		try {
			return super.getResource(uri, loadOnDemand);
		} finally {
			inGetResource.set(oldValue);
		}
	}

	boolean isInGetResource() {
		return inGetResource.get() == Boolean.TRUE;
	}

	@Override
	public Resource createResource(URI uri, String contentType) {
		Resource resource = super.createResource(uri, contentType);
		initView(resource);
		return resource;
	}

	@Override
	protected void demandLoad(Resource resource) throws IOException {
		URI uri = resource.getURI();
		if (CDOUtils.isCDOURI(uri)) {
			// XML options not applicable to CDO resources
			resource.load(null);

			resourceLoadedHook(resource);
		} else {
			super.demandLoad(resource);
		}

		initView(resource);
	}

	@Override
	protected Resource setResourceOptions(Resource resource) {
		if (resource instanceof CDOResource) {
			return resource;
		}

		return super.setResourceOptions(resource);
	}

	protected void resourceLoadedHook(Resource resource) {
		for (Diagram next : Iterables.filter(resource.getContents(), Diagram.class)) {

			DawnDiagramUpdater.initializeElement(next);
		}
	}

	public CDOView getCDOView() {
		return view;
	}

	@Override
	public void createModels(URI newURI) {
		super.createModels(newURI);
	}

	@Override
	public void loadModels(URI uri) throws ModelMultiException {

		// If Eclipse was previously shut down with editors still open, these editors will be reopened now.
		// This does not go through PapyrusCDOEditorOpener, so we have no chance there to open the needed checkout.
		CDOCheckout checkout = CDOExplorerUtil.getCheckout(uri);
		if (checkout != null) {
			checkout.open();
		}

		super.loadModels(uri);

		// Did any of the models create a new resource? (the DiModel and/or HistoryModel may have done)
		// If so, commit them now so that they don't dangle if we later close the editor without saving.
		out: for (CDOResource next : Iterables.filter(getResources(), CDOResource.class)) {
			switch (next.cdoState()) {
			case NEW:
			case DIRTY:
				try {
					((CDOTransaction) getCDOView()).commit(new NullProgressMonitor());
				} catch (CommitException e) {
					Activator.log.error("Commit implicitly created resources failed.", e); //$NON-NLS-1$
				}
				break out;
			default:
				// Pass
				break;
			}
		}
	}

	protected void initView(Resource resource) {
		if (view == null && resource instanceof CDOResource) {
			view = ((CDOResource) resource).cdoView();
			if (view != null) {
				CDOViewSet viewSet = view.getViewSet();

				for (CDOView v : viewSet.getViews()) {
					EventUtil.addUniqueListener(v, invalidationListener);
				}

				viewSet.eAdapters().add(viewSetAdapter);

				// Don't clear adapters of the resource because ECrossReferenceAdapters would try to crawl the contents.
				viewSet.setDefaultClearAdapterPolicy((object, adapter) -> !(object instanceof CDOResourceNode));
			}
		}
	}

	@Override
	public void unload() {
		boolean logError = view == null || !view.isClosed();

		try {
			if (view != null) {
				CDOViewSet viewSet = view.getViewSet();
				viewSet.eAdapters().remove(viewSetAdapter);

				for (CDOView v : viewSet.getViews()) {
					EventUtil.removeListener(v, invalidationListener);
				}
			}

			unloading = true;
			super.unload();
		} catch (Exception ex) {
			if (logError) {
				Activator.log.error(ex);
			}
		} finally {
			unloading = false;

			// Unload and remove the resources that were omitted in super.unload()
			// because of CDOAwareModelSet.SafeResourceList.iterator().
			EList<Resource> resources = getResources();
			for (Resource resource : resources) {
				resource.unload();
			}

			resources.clear();
		}
	}


	@Override
	public boolean isUserModelResource(URI uri) {
		return super.isUserModelResource(uri) || CDOUtils.isCDOURI(uri);
	}

	boolean isDirty(Resource resource) {
		return (resource instanceof CDOResource) && DIRTY_STATES.contains(((CDOResource) resource).cdoState());
	}

	@Override
	protected void cleanModelSet() {
		Set<URI> toDelete = getResourcesToDeleteOnSave();
		for (Iterator<Resource> iter = getResources().iterator(); iter.hasNext();) {
			Resource next = iter.next();

			// Can't remove a dirty CDO resource
			if (toDelete.contains(next.getURI()) && (!(next instanceof CDOResource) || FSMUtil.isClean((CDOResource) next) || FSMUtil.isTransient((CDOResource) next))) {
				iter.remove();
			}
		}
	}

	@Override
	protected void handleResourcesToDelete() {
		final int initialCount = getResourcesToDeleteOnSave().size();

		super.handleResourcesToDelete();

		// any changes made by resource deletions?
		CDOView view = getCDOView();
		if ((view instanceof CDOTransaction) && (getResourcesToDeleteOnSave().size() < initialCount)) {
			try {
				((CDOTransaction) view).commit();
			} catch (CommitException e) {
				Activator.log.error("Failed to commit resource deletions.", e); //$NON-NLS-1$
			}
		}
	}

	@Override
	protected boolean validateDeleteResource(URI uri) {
		boolean result = true;

		Resource resource = getResource(uri, false);

		// if it was meant to be deleted, an attempt would have been made to remove it. That may
		// have been prevented if it was dirty. If it isn't now, then delete it. Otherwise,
		// block again
		if ((resource != null) && isDirty(resource)) {
			result = false;
			Activator.log.warn("Attempt to delete a dirty CDO resource: " + uri); //$NON-NLS-1$
		}

		return result;
	}

	@Override
	public boolean deleteResource(URI uri) {
		Resource res = getResource(uri, false);
		boolean result = res instanceof CDOResource;

		if (!result) {
			// not a CDO resource. Default behaviour
			result = super.deleteResource(uri);
		} else {
			try {
				res.delete(null);
				if (res.getResourceSet() != null) {
					res.unload();
					res.getResourceSet().getResources().remove(res);
				}
			} catch (IOException e) {
				Activator.log.error(e);
			}
		}

		return result;
	}

	@Override
	public void save(IProgressMonitor monitor) throws IOException {
		CDOView view = getCDOView();
		CDOTransaction transaction = null;

		Collection<CDOObject> updates;
		if ((view instanceof CDOTransaction) && view.isDirty()) {
			// collect updated objects to post-process for cross-unit references
			transaction = (CDOTransaction) view;
			updates = ImmutableList.<CDOObject> builder() //
					.addAll(transaction.getNewObjects().values()) //
					.addAll(transaction.getDirtyObjects().values()) //
					.build();
		} else {
			updates = Collections.emptyList();
		}

		SubMonitor sub = SubMonitor.convert(monitor, updates.isEmpty() ? 1 : 2);

		super.save(sub.newChild(1));

		if (!updates.isEmpty()) {
			CDOControlModeParticipant control = new CDOControlModeParticipant();
			CDOControlModeParticipant.IUpdate run = CDOControlModeParticipant.IUpdate.EMPTY;

			for (CDOObject next : updates) {
				// Resources don't have cross-references that we need to refactor
				if (!(next instanceof CDOResourceNode)) {
					EObject object = CDOUtil.getEObject(next);
					if (object != null) {
						for (EReference xref : object.eClass().getEAllReferences()) {
							// do include containment references because we may have added a model
							// element and controlled it in the same transaction
							if (xref.isChangeable() && !xref.isDerived() && !xref.isTransient()) {
								run = run.chain(control.getProxyCrossReferencesUpdate(object, xref));
							}
						}
					}
				}
			}

			if (!run.isEmpty()) {
				run.apply();

				try {
					transaction.commit(sub.newChild(1));
				} catch (CommitException e) {
					Activator.log.error("Follow-up commit after save failed.", e); //$NON-NLS-1$
				}
			} else {
				sub.done();
			}
		}
	}

	//
	// Nested types
	//

	/**
	 * CDO doesn't permit resources to be removed from the resource set if they are {@linkplain CDOState#DIRTY dirty},
	 * so this specialized list prevents that.
	 * 
	 * It also prevents early removal of CDOResources during {@link CDOAwareModelSet#unload() unload()}.
	 * 
	 * @author Eike Stepper
	 */
	private class SafeResourceList extends ResourcesEList<Resource> {
		private static final long serialVersionUID = 1L;

		public SafeResourceList() {
		}

		@Override
		public boolean remove(Object object) {
			if (object instanceof CDOResource && isDirty((CDOResource) object)) {
				return false;
			}

			return super.remove(object);
		}

		@Override
		public Iterator<Resource> iterator() {
			Iterator<Resource> delegate = super.iterator();

			if (unloading) {
				return new AbstractFilteredIterator<Resource>(delegate) {
					@Override
					protected boolean isValid(Resource resource) {
						return !(resource instanceof CDOResource);
					}
				};
			}

			return delegate;
		}
	}

	/**
	 * @author Eike Stepper
	 */
	private final class ViewSetAdapter extends AdapterImpl {

		public ViewSetAdapter() {
		}

		@Override
		public void notifyChanged(Notification notification) {
			switch (notification.getEventType()) {
			case Notification.ADD:
				CDOView newView = (CDOView) notification.getNewValue();
				EventUtil.addUniqueListener(newView, invalidationListener);
				break;

			case Notification.REMOVE:
				CDOView oldView = (CDOView) notification.getOldValue();
				EventUtil.removeListener(oldView, invalidationListener);
				break;
			}
		}
	}

	/**
	 * @author Eike Stepper
	 */
	private final class InvalidationListener implements IListener {

		public InvalidationListener() {
		}

		@Override
		public void notifyEvent(IEvent event) {
			if (event instanceof CDOViewInvalidationEvent) {
				TransactionalEditingDomain domain = getTransactionalEditingDomain();
				if (domain instanceof CDOAwareTransactionalEditingDomain) {
					((CDOAwareTransactionalEditingDomain) domain).fireResourceSetChanged((CDOViewInvalidationEvent) event);
				}
			}
		}
	}
}
